#ifndef _FLE_MDIWORKSPACE_H_
#define _FLE_MDIWORKSPACE_H_

#include <fltk/Fl_Window.h>
#include <fltk/Fl_Scrollbar.h>
#include <fltk/Fl_Menu_Bar.h>
#include <fltk/fl_draw.h>
#include <fltk/Fl.h>
#include <fltk/x.h>

#include <stdio.h>

#include "EDE.h"

#ifndef _WIN32
#include <X11/Xlib.h>
#else
#include <windows.h>
#endif

class FLE_MdiWindow;
class FLE_Workspace;

#define FLE_MDI_WINDOW 0x01000000

typedef Fl_Group FLE_WorkType;
typedef Fl_Window FLE_ViewportType;

 /*! 
   *  Viewport is MDI window container. It handles most common MDI needs like:
   *    - Redrawing all in once
   *	- Relayouting all in once
   *	- Attach / Detach window
   *	- Always on top window
   *	- Maximum window
   *	
   *  It is possible to get min/close buttons to menubar by setting menu pointer, using menu() function.
   *  Viewport will add new child to that menu and resize it always to right of the menubar.
   *
   *  Viewport can be "scrolled", using scrolld* functions. These will only move all childrens in layout, according dx and dy.
   *  See FLE_WorkSpace for information about that.
   *
   *  Basically Viewport is only used, if you dont want nothing special in mdi container, like scrollbars.
   *  For more container advanced See FLE_WorkSpace
   */
class FLE_MdiViewport : public FLE_ViewportType
{	
public:
    /*!
      Constructs widget with given coordinates.
    */
	FLE_MdiViewport(int x, int y, int w, int h, const char *label = 0);

	/*!
      Deletes widget and all its children.
    */
	virtual ~FLE_MdiViewport() { }
	
    /*!
		Force to redraw all MDI windows
    */
	void redraw_all();
	/*!
		Force to relayout all MDI windows
    */	
	void relayout_all();

	/*!
		@param caption As a string to match with window caption.
		Finds MDI window by titlebar caption. 
		
		Returns pointer to window if found, otherwise 0.
	*/
    FLE_MdiWindow *find(const char *caption);

	/*!
		@param win As window to attach.
		Attachs MDI window.		
	*/
    void attach(FLE_MdiWindow *win);
	/*!
		@param win As window to detach.
		Detachs MDI window. (makes it toplevel)		
	*/
    void detach(FLE_MdiWindow *w);

	/*!
		Closes all MDI windows.
	*/
    void close_all();

	/*!
		Returns "Always On Top" MDI window.
		@see aot(FLE_MdiWindow *win)
	*/
    FLE_MdiWindow *aot() { return _aot; }
	/*!
		@param win As MDI window to set.
		Sets "Always On Top" MDI window. And relayouts viewport.		
		@see aot()
	*/
    void aot(FLE_MdiWindow *win) { _aot = win; relayout(); }

	/*!		
		Returns "Maximum" MDI window, if there is one. 
		
		Otherwise returns 0
		@see maximum(FLE_MdiWindow *w)
	*/
    FLE_MdiWindow *maximum() { return _max; }
	/*!		
		@param win As MDI window to set.
		Sets "Maximum" MDI window.
		@see maximum()
	*/	
    void maximum(FLE_MdiWindow *win);

	/*!
		@param win As a MDI window to set.
		Activate and set MDI window to top of the window stack.
		@see top()
	*/
    void top(FLE_MdiWindow *win);
	/*!
		Returns current active window.
		@see top(FLE_MdiWindow *win)
	*/
    FLE_MdiWindow *top() { return _top; }

	/*!
		Returns menubar, where min/close buttons are. when MDI window maximized.
		Returns 0 if no menubar setted.
		@see menu(Fl_Menu_Bar *menu)
	*/
    Fl_Menu_Bar *menu() { return _menu; }
	/*!
		Sets menubar, where min/close buttons are. when MDI window maximized.
		@see menu()
	*/
    void menu(Fl_Menu_Bar *menu) { _menu = menu; }
    
	/*!
		Returns number of all visible MDI windows.
	*/
	int cnt_windows();

	/*!
		Cycles windows, sets next window to active.
		@see top(FLE_MdiWindow *win)
	*/	
	void cycle_windows();

	/*!
		@param dy As a +/- pixels to move all children
		This will move childrens vertically by dy, when layout gets called next time.
		Please, DO NOT use this, unless you are knowing what yuo are doing.
		@see scrolldx(int dx)
	*/
	void scrolldy(int dy) { _scrolldy+=dy; }
	/*!
		@param dx As a +/- pixels to move all children
		This will move childrens horizontally by dx, when layout gets called next time.
		Please, DO NOT use this, unless you are knowing what yuo are doing.
		@see scrolldy(int dy)
	*/
	void scrolldx(int dx) { _scrolldx+=dx; }

	/*!
		This is called, when widget changes its size or relayout called by user.
		This makes sure that aot window is actually always on top. And layouts minimized windows
		to left-bottom corner of viewport. If you override this, you should call this at the beginning
		of function.
	*/
    virtual void layout();
	/*!
		@param ev As a FLTK event number passed to function
		This handles only keyboard clicks. If you override this and want ALT+TAB cycles windows
		you should call this also.
	*/
    virtual int handle(int ev);
	/*!
		Draws all MDI windows and other childrens.
	*/
    virtual void draw();

private:
	friend class FLE_MdiWindow;

    FLE_MdiWindow *_aot; //Always On Top
    FLE_MdiWindow *_max; //Maximized
    FLE_MdiWindow *_top; //Top most, active

	// For scrolling:
	static void cb_draw_clip(void*,int,int,int,int);
	void draw_clip(int,int,int,int);
	int _scrolldx, _scrolldy;
    
	Fl_Menu_Bar *_menu;
};

/*! 
   *  Workspace is advanced MDI window container. It handles some special MDI needs like:
   *    - Tiling (V/H)
   *	- Cascade
   *	- Scrollbars if window(s) outside of viewport
   *
   *  Workspace normally used in every program as a MDI container, not FLE_MdiViewport. 
   *  This can handle larger workareas very easily, e.g. viewport is only 640x480, can 
   *  whole workspace be 2048x1024. 
   *
   *  Workspace wraps most of viewports functions, so this can be used like a viewport. 
   *  It also provides backward compatibility.
   */
class FLE_WorkSpace : public FLE_WorkType  {    
public:
	/*!
      Constructs widget with given coordinates.
    */
    FLE_WorkSpace(int x, int y, int w, int h, const char *label = 0);
	/*!
      Deletes widget and all its children.
    */
    virtual ~FLE_WorkSpace();

	//@{
    /*!
		Wraps viewport functionality. And provides backward compatibility.
		@see FLE_MdiViewport
	*/
    void begin() { viewport()->begin(); }
    void end()   { viewport()->end(); }

    void redraw_all()   { viewport()->redraw_all(); }
    void relayout_all() { viewport()->relayout_all(); }

    FLE_MdiWindow *find(const char *caption) { return viewport()->find(caption); }

    void attach(FLE_MdiWindow *win) { viewport()->attach(win); }
    void detach(FLE_MdiWindow *win) { viewport()->detach(win); }

    void close_all() { _viewport->close_all(); }

    FLE_MdiWindow *aot() { return viewport()->aot(); }
    void aot(FLE_MdiWindow *win) { viewport()->aot(win); }

    FLE_MdiWindow *maximum() { return viewport()->maximum(); }
    void maximum(FLE_MdiWindow *win) { viewport()->maximum(win); }

    void top(FLE_MdiWindow *win) { viewport()->top(win); }
    FLE_MdiWindow *top() { return viewport()->top(); }	

    Fl_Menu_Bar *menu() { return viewport()->menu(); }
    void menu(Fl_Menu_Bar *menu) { viewport()->menu(menu); }
	//@}

	/*!
		Cascade layout all visible MDI windows.
	*/
    void cascade();

	/*! 
		Vertical tile layout all visible MDI windows.
	*/
    void tileV();
	/*!
		Horizontal tile layout all visible MDI windows.
	*/
    void tileH();
	/*!
		Wraps horizontal tile layout.
		@see tileH()
	*/
	void tile() { tileH();}	

	/*!
		@param val Is either True or False.

		Moves viewport position, if focused window is outside of viewport visible area.
		@see focus_moves_pos()
	*/
	void focus_moves_pos(bool val);
	/*!
		Returns value setted by focus_moves_pos(bool val).
		@see focus_moves_pos(bool val)
	*/
	bool focus_moves_pos() { return move_pos; }

	FLE_MdiViewport *viewport() { return _viewport; }

	/*! 
		values for type()		
     */
	enum { 
		HORIZONTAL = 1,			/**< Show horizontal scrollbar, if needed */
		VERTICAL = 2,			/**< Show vertical scrollbar, if needed */
		BOTH = 3,				/**< Show both scrollbars, if needed */
		ALWAYS_ON = 4,			
		HORIZONTAL_ALWAYS = 5,	/**< Horizontal scrollbar always visible */
		VERTICAL_ALWAYS = 6,	/**< Vertical scrollbar always visible */
		BOTH_ALWAYS = 7			/**< Both scrollbars always visible  */
	};

	/*!
		Moves viewport to show active window, if outside of workarea.
		@see show_window(FLE_MdiWindow *win)
	*/
	void show_window() { show_window(viewport()->top()); }
	/*!
		@param win As a MDI window to show
		Moves viewport to show window, if outside of workarea.
		@see show_window()
	*/
	void show_window(FLE_MdiWindow *win);

	/*!
		Returns X position of viewport
		@see yposition() and position(int x, int y)
	*/
	int xposition() const {return xposition_;}
	/*!
		Returns Y position of viewport
		@see xposition() and position(int x, int y)
	*/
	int yposition() const {return yposition_;}
	/*!
		Sets X and Y position of viewport and moves it to that position straight away.
		@see yposition() and xposition()
	*/
	void position(int x, int y);	

	/*!
		This is called, when widget changes its size or relayout called by user.
		Keeps track of visibility of scrollbars. Calculates viewport size.
	*/
    virtual void layout();
	/*!
		Redirect events to scrollbars, if needed.
	*/
    virtual int handle(int ev);
	/*!
		Draws scrollbars and call FLE_MdiViewport to draw MDI windows
	*/
    virtual void draw();

private:
	static void cb_hscrollbar(Fl_Widget *o, void *d);
	static void cb_vscrollbar(Fl_Widget *o, void *d);

	void bbox(int&,int&,int&,int&);	

	FLE_MdiViewport *_viewport;    

	int xposition_, yposition_;
	int layoutdx, layoutdy;

	Fl_Scrollbar *vscrollbar;
	Fl_Scrollbar *hscrollbar;

	bool move_pos;
};

#endif
